from __future__ import with_statement

import es
import os
import sqlite3
import installlib
# import cmdlib
import gamethread
# import zipfile
import time
import re
from operator import truediv # I am not a real fan of __future__.division



update_check_enabled = True



info = es.AddonInfo()
info.basename = 'bansaver'
info.name = "BanSaver"
info.author = "Brainsucker"
info.description = "Saves NON-Permanent bans. So the bans will not expire after a server restart."
info.url = "http://addons.eventscripts.com/addons/view/bansaver"
info.version = "1.3.3"


def updatecheck():
    installer = installlib.getInstaller('bansaver')
    
    us = installer.update()
    
    if us == installer.STATUS_SUCCESSFUL:
        es.dbgmsg(0, '[%s] Updated to new Version (v%s). Please reload the Addon or restart your Server.' % (info['name'], installer.addon.currentversion))
    
    
addonpath = es.getAddonPath('bansaver')
dbpath = os.path.join(addonpath, 'banned.sql')
# backup_basepath = os.path.join(addonpath, 'backups')

bantime_regex = re.compile(r'for ([\d\.]+) minutes', re.I)

# sql_db = sqlite3.connect(dbpath)
# sql_cur = sql_db.cursor()

class SQLiteDB(object):
    def __init__(self):
        self.connection = sqlite3.connect(dbpath)
        self.cursor = self.connection.cursor()
        
        self.execute("CREATE TABLE IF NOT EXISTS banned_user (steamid TEXT UNIQUE, ip TEXT UNIQUE, expiredate REAL)")
        
        self.removeExpired()
        
        self.registerBans()
        
    def removeExpired(self):
        self.execute("DELETE FROM banned_user WHERE expiredate<=?", (time.time(), ))
        self.commit()
        
    def registerBans(self):
        delay_time = 0.05 # Delay all bans a bit, when loading the addon. This will avoid a buffer overflow.
        result = self.execute("SELECT * FROM banned_user")
        for id, ip, ed in result:
            delay_time += 0.05
            remaining_seconds = ed - time.time()
            remaining_minutes = truediv(remaining_seconds, 60)
            if id:
                gamethread.delayed(delay_time, es.server.queuecmd, ('banid %f %s' % (remaining_minutes, id)))
                
            if ip:
                gamethread.delayed(delay_time, es.server.queuecmd, ('addip %f %s' % (remaining_minutes, ip)))
                
    def addID(self, id, expiry):
        self.execute("INSERT OR REPLACE INTO banned_user(steamid, expiredate) VALUES(?, ?)", (id, expiry))
        self.commit()
    
    def addIP(self, ip, expiry):
        self.execute("INSERT OR REPLACE INTO banned_user(ip, expiredate) VALUES(?, ?)", (ip, expiry))
        self.commit()
                
    def removeID(self, id):
        self.execute("DELETE FROM banned_user WHERE steamid=?", (id,))
        self.commit()
        
    def removeIP(self, ip):
        self.execute("DELETE FROM banned_user WHERE ip=?", (ip,))
        self.commit()
        
    def execute(self, query, args=None):
        if args:
            return self.cursor.execute(query, args)
        else:
            return self.cursor.execute(query)
            
    def commit(self):
        self.connection.commit()
        
    def close(self):
        self.cursor.close()
        
db = SQLiteDB()


# def clear_db(args):
    # if args[0].lower() == 'help':
        # es.dbgmsg(0, 'Argument Syntax: <command> [backup]')
        # es.dbgmsg(0, '1. Argument: Defines if you want the script to create a backup of all bans.')
        # return
        
        
    # if args[0].lower() == 'backup':
        # bfp = os.path.join(backup_basepath, os.sep.join(time.strftime('%Y %B %d').split()))
        # if not os.path.isdir(bfp):
            # os.makedirs(bfp)
        
        # bfname = time.strftime('%H%M%S') + '.zip'
        # full_bfpath = os.path.join(bfp, bfname)
        # zfn = '%s.zip' % bfname
        # zf = zipfile.ZipFile(full_bfpath, mode='w')
        # zf.write(dbpath)
        # zf.close()
        
        
    # sql_cur.execute("DELETE FROM banned_user")
    # sql_db.commit()
    

def load():
    
    es.ServerVar(info["basename"]+'_ver', info["version"]).makepublic()
    
    
    # cmdlib.registerServerCommand('bansaver_cleardb', clear_db, 'Purges whole bansaver database. (USE WITH CAUTION!)')
    
def unload():
    db.commit()
    db.close()
    # cmdlib.unregisterServerCommand('bansaver_cleardb')
    

def es_map_start(ev):
    if update_check_enabled:
        updatecheck()


def server_addban(ev):
    steamid = ev["networkid"]
    ip = ev['ip']
    dur = ev["duration"]
    if dur == 'permanently':
    
        if steamid:
            db.removeID(steamid)
    
        if ip:
            db.removeIP(ip)
            
    else:
        m = bantime_regex.match(dur)
        if m:
            bantime = float(m.group(1))
            ed = time.time() + (bantime * 60)
            if steamid:
                db.addID(steamid, ed)
                    
            if ip:
                db.addIP(ip, ed)
            
def server_removeban(ev):
    steamid = ev["networkid"]
    ip = ev['ip']
    if steamid:
        db.removeID(steamid)
    if ip:
        db.removeIP(ip)